@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2010-2013 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2010-2014 Matthias Klumpp <matthias@tenstral.net>
  *                    2011 Richard Hughes <richard@hughsie.com>
  * Licensed under the GNU Lesser General Public License Version 3
  *
@@ -23,6 +23,7 @@
 #include <locale.h>
 #include <packagekit-glib2/packagekit.h>
 #include <plugin/packagekit-plugin.h>
+#include <appstream.h>
 
 #include "listaller_internal.h"
 #include "li-plugin-utils.h"
@@ -31,10 +32,9 @@ struct PkPluginPrivate {
 	PkTransaction		*current_transaction;
 	PkBackendJob		*internal_job; /* job to communicate with the backend - plugin->job is used for frontend communication */
 	ListallerManager	*mgr;
-	ListallerSetupSettings	*setup_settings;
-	PkResults		*backend_results;
-	GMainLoop		*loop;
-	gboolean		error_set;
+	PkResults			*backend_results;
+	GMainLoop			*loop;
+	gboolean			error_set;
 };
 
 static void pk_plugin_reset_backend_jobs (PkPlugin *plugin);
@@ -61,11 +61,9 @@ pk_plugin_reset (PkPlugin *plugin)
 static void
 pk_plugin_skip_native_backend (PkPlugin *plugin)
 {
-	PkExitEnum exit;
-	exit = pk_backend_job_get_exit_code (plugin->job);
 	g_debug ("Skipping native backend.");
 
-	/* only skip transaction if we don't have an error already */
+	/* only skip transaction if an error is not already set */
 	if (!pk_backend_job_get_is_error_set (plugin->job)) {
 		pk_backend_job_set_exit_code (plugin->job, PK_EXIT_ENUM_SKIP_TRANSACTION);
 	}
@@ -147,9 +145,11 @@ void
 pk_listaller_get_details (PkPlugin *plugin, gchar **package_ids)
 {
 	const gchar *description;
+	const gchar *summary;
 	ListallerAppLicense license;
 	const gchar *url;
 	ListallerAppItem *app;
+	AsComponent *cpt;
 	guint i;
 
 	g_debug ("listaller: running get_details ()");
@@ -160,19 +160,23 @@ pk_listaller_get_details (PkPlugin *plugin, gchar **package_ids)
 		/* update AppItem with database data */
 		listaller_manager_refresh_appitem_data (plugin->priv->mgr, &app);
 
-		description = listaller_app_item_get_description (app);
+		cpt = listaller_app_item_get_metainfo (app);
+		description = as_component_get_description (cpt);
+		summary = as_component_get_summary (cpt);
 		listaller_app_item_get_license (app, &license);
-		url = listaller_app_item_get_website (app);
+		url = as_component_get_url (cpt, AS_URL_KIND_HOMEPAGE);
 
-		//TODO: Fetch size of installed application too
+		/* TODO: Fetch size of installed application too */
 
 		/* emit */
-		pk_backend_job_details (plugin->job, package_ids[i],
-					license.name,
-					PK_GROUP_ENUM_UNKNOWN,
-					description,
-					url,
-					0);
+		pk_backend_job_details (plugin->job,
+						package_ids[i],
+						summary,
+						license.name,
+						PK_GROUP_ENUM_UNKNOWN,
+						description,
+						url,
+						0);
 	};
 }
 
@@ -212,6 +216,7 @@ static void
 listaller_application_cb (GObject *sender, ListallerAppItem *item, PkPlugin *plugin)
 {
 	gchar *package_id;
+	AsComponent *cpt;
 
 	package_id = pk_listaller_pkid_from_appitem (item);
 	if (package_id == NULL) {
@@ -220,9 +225,11 @@ listaller_application_cb (GObject *sender, ListallerAppItem *item, PkPlugin *plu
 	}
 	g_debug ("new app found -> %s (%s)", listaller_app_item_get_appid (item), package_id);
 
+	cpt = listaller_app_item_get_metainfo (item);
+
 	/* emit */
 	pk_backend_job_package (plugin->job, PK_INFO_ENUM_INSTALLED, package_id,
-			    listaller_app_item_get_summary (item));
+			    as_component_get_summary (cpt));
 
 	g_free (package_id);
 }
@@ -288,7 +295,7 @@ listaller_progress_cb (GObject *sender, ListallerProgressItem *item, PkPlugin *p
 
 	if (prog_type == LISTALLER_PROGRESS_ENUM_ITEM_PROGRESS) {
 		const gchar *item_id = listaller_progress_item_get_item_id (item);
-		// TODO: Implement better status handling
+		/* TODO: Implement better status handling */
 		if (value > 0)
 			pk_backend_job_set_item_progress (plugin->job, item_id, PK_STATUS_ENUM_RUNNING, value);
 	}
@@ -331,6 +338,7 @@ pk_listaller_install_file (PkPlugin *plugin, const gchar *filename)
 	ListallerSetup *setup;
 	gchar* package_id;
 	ListallerAppItem *app = NULL;
+	AsComponent *cpt;
 
 	setup = listaller_setup_new (filename);
 	g_signal_connect (setup, "error-code",
@@ -359,10 +367,11 @@ pk_listaller_install_file (PkPlugin *plugin, const gchar *filename)
 	if (package_id == NULL) {
 		g_debug ("listaller: <error> Unable to build package-id from app-id!");
 	} else if (!pk_backend_job_get_is_error_set (plugin->job)) {
+		cpt = listaller_app_item_get_metainfo (app);
 		/* emit */
 		pk_backend_job_package (plugin->job, PK_INFO_ENUM_INSTALLED,
 					package_id,
-					listaller_app_item_get_summary (app));
+					as_component_get_summary (cpt));
 		g_free (package_id);
 	}
 	g_object_unref (app);
@@ -409,7 +418,8 @@ pk_listaller_get_updates (PkPlugin *plugin)
 	GeeArrayList *available_updates;
 	ListallerUpdateItem *uitem;
 	ListallerAppItem *app;
-	ListallerDepModule *dep_mod;
+	ListallerDependency *dep;
+	AsComponent *cpt;
 	gchar *package_id;
 	GType swtype;
 	gpointer ptr;
@@ -441,11 +451,12 @@ pk_listaller_get_updates (PkPlugin *plugin)
 			}
 
 			g_debug ("Emitting update package: %s", package_id);
+			cpt = listaller_app_item_get_metainfo (app);
 			pk_backend_job_package (plugin->job, PK_INFO_ENUM_NORMAL,
 						package_id,
-						listaller_app_item_get_summary (app));
+						as_component_get_summary (cpt));
 			g_free (package_id);
-		} else if (swtype == LISTALLER_DEP_TYPE_MODULE) {
+		} else if (swtype == LISTALLER_TYPE_DEPENDENCY) {
 			// TODO
 		} else {
 			g_warning ("Found UpdateItem with invalid software type: %s", g_type_name (swtype));
@@ -595,52 +606,12 @@ pk_plugin_reset_backend_jobs (PkPlugin *plugin)
 }
 
 /**
- * pk_backend_job_request_whatprovides_cb:
- *
- * Helper method for a Listaller native PkBackend proxy to do a what-provides request.
- **/
-static PkResults*
-pk_backend_job_request_whatprovides_cb (PkBitfield filters,
-				PkProvidesEnum provides,
-				gchar** search,
-				PkPlugin *plugin)
-{
-	PkResults *results;
-
-	/* query the native backend for a package provinding X */
-	if (plugin == NULL)
-		g_debug ("PLUGIN was NULL!");
-
-	g_debug ("Running what-provides on native backend!");
-
-	/* prepare for native backend call */
-	pk_plugin_reset (plugin);
-
-	/* query the native backend */
-	pk_backend_what_provides (plugin->backend,
-				   plugin->priv->internal_job,
-				   filters,
-				   provides,
-				   search);
-
-	/* wait for finished */
-	g_main_loop_run (plugin->priv->loop);
-
-	results = plugin->priv->backend_results;
-	pk_plugin_reset_backend_jobs (plugin);
-
-	g_debug ("Results exit code is %s", pk_exit_enum_to_string (pk_results_get_exit_code (results)));
-
-	return results;
-}
-
-/**
- * pk_backend_job_request_installpackages_cb:
+ * pk_backend_job_request_install_packages_cb:
  *
  * Helper method for a Listaller native PkBackend proxy to do a install-packages request.
  **/
 static PkResults*
-pk_backend_job_request_installpackages_cb (PkBitfield transaction_flags,
+pk_backend_job_request_install_packages_cb (PkBitfield transaction_flags,
 					gchar **package_ids,
 					PkPlugin *plugin)
 {
@@ -668,6 +639,74 @@ pk_backend_job_request_installpackages_cb (PkBitfield transaction_flags,
 	return results;
 }
 
+/**
+ * pk_backend_job_request_resolve_packages_cb:
+ *
+ * Helper method for a Listaller native PkBackend proxy to perform a resolve request.
+ **/
+static PkResults*
+pk_backend_job_request_resolve_packages_cb (PkBitfield transaction_flags,
+					gchar **package_ids,
+					PkPlugin *plugin)
+{
+	PkResults *results;
+
+	g_debug ("Running resolve-packages on native backend!");
+
+	/* prepare the backend */
+	pk_plugin_reset (plugin);
+
+	/* query the native backend */
+	pk_backend_resolve (plugin->backend,
+				      plugin->priv->internal_job,
+				      transaction_flags,
+				      package_ids);
+
+	/* wait for finished */
+	g_main_loop_run (plugin->priv->loop);
+
+	results = plugin->priv->backend_results;
+	pk_plugin_reset_backend_jobs (plugin);
+
+	g_debug ("Results exit code is %s", pk_exit_enum_to_string (pk_results_get_exit_code (results)));
+
+	return results;
+}
+
+/**
+ * pk_backend_job_request_search_files_cb:
+ *
+ * Helper method for a Listaller native PkBackend proxy to perform a search-files request.
+ **/
+static PkResults*
+pk_backend_job_request_search_files_cb (PkBitfield transaction_flags,
+					gchar **files,
+					PkPlugin *plugin)
+{
+	PkResults *results;
+
+	g_debug ("Running resolve-packages on native backend!");
+
+	/* prepare the backend */
+	pk_plugin_reset (plugin);
+
+	/* query the native backend */
+	pk_backend_search_files (plugin->backend,
+				      plugin->priv->internal_job,
+				      transaction_flags,
+				      files);
+
+	/* wait for finished */
+	g_main_loop_run (plugin->priv->loop);
+
+	results = plugin->priv->backend_results;
+	pk_plugin_reset_backend_jobs (plugin);
+
+	g_debug ("Results exit code is %s", pk_exit_enum_to_string (pk_results_get_exit_code (results)));
+
+	return results;
+}
+
 /**
  * pk_plugin_transaction_started:
  *
@@ -689,6 +728,13 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 	/* set the transaction */
 	plugin->priv->current_transaction = transaction;
 
+	/* we use an internal job to communicate with the native backend. plugin->job is used for client communication */
+	if (plugin->priv->internal_job == NULL) {
+		GKeyFile *conf;
+		conf = pk_transaction_get_conf (plugin->priv->current_transaction);
+		plugin->priv->internal_job = pk_backend_job_new (conf);
+	}
+
 	plugin->priv->error_set = FALSE;
 
 	/* reset all jobs connected to the backend (public/private) */
@@ -696,19 +742,15 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 	pk_backend_job_set_status (plugin->job, PK_STATUS_ENUM_SETUP);
 
 	/* create a backend proxy and connect it, so Listaller can acces parts of PkBackend */
-	pkbproxy = listaller_pk_backend_proxy_new ();
-	listaller_pk_backend_proxy_set_what_provides (pkbproxy,
-						(ListallerPkBackendProxyWhatProvidesCB) pk_backend_job_request_whatprovides_cb,
-						plugin);
-	listaller_pk_backend_proxy_set_install_packages (pkbproxy,
-						(ListallerPkBackendProxyInstallPackagesCB) pk_backend_job_request_installpackages_cb,
-						plugin);
+	pkbproxy = listaller_pk_backend_proxy_new ((ListallerPkBackendProxyResolvePackagesCB) pk_backend_job_request_resolve_packages_cb, plugin,
+												(ListallerPkBackendProxySearchFilesCB) pk_backend_job_request_search_files_cb, plugin,
+												(ListallerPkBackendProxyInstallPackagesCB) pk_backend_job_request_install_packages_cb, plugin);
 	listaller_set_backend_proxy (pkbproxy);
 
 	/* get transaction role */
 	role = pk_transaction_get_role (transaction);
 
-	/* if we're only simulation, skip Listaller packages */
+	/* if we're only simulating, skip Listaller packages */
 	if (pk_bitfield_contain (pk_transaction_get_transaction_flags (transaction),
 				   PK_TRANSACTION_FLAG_ENUM_SIMULATE)) {
 
@@ -718,7 +760,7 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 									full_paths);
 
 			/* We have Listaller packages, so skip this! */
-			/* FIXME: This needs to be smarter - backend needs to Simulate() with remaining pkgs */
+			/* TODO: This needs to be smarter - backend needs to Simulate() with remaining pkgs */
 			if (data != NULL)
 				pk_plugin_skip_native_backend (plugin);
 		} else {
@@ -736,11 +778,10 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 	}
 
 	/* set locale so Listaller messages are translated */
-        if (locale = pk_backend_job_get_locale (plugin->job)) {
-            setenv ("LANGUAGE", locale, 1);
-            setenv ("LANG", locale, 1);
-        }
-        g_free (locale);
+	locale = pk_backend_job_get_locale (plugin->job);
+	setenv ("LANGUAGE", locale, 1);
+	setenv ("LANG", locale, 1);
+	g_free (locale);
 
 	g_debug ("Processing transaction");
 
@@ -894,7 +935,7 @@ pk_plugin_transaction_finished_results (PkPlugin *plugin,
 const gchar *
 pk_plugin_get_description (void)
 {
-	// TODO: Think of a better description
+	/* TODO: Think of a better description */
 	return "Listaller support for PackageKit";
 }
 
@@ -909,10 +950,6 @@ pk_plugin_initialize (PkPlugin *plugin)
 	plugin->priv->mgr = listaller_manager_new (TRUE);
 	plugin->priv->loop = g_main_loop_new (NULL, FALSE);
 	plugin->priv->backend_results = pk_results_new ();
-	plugin->priv->setup_settings = NULL;
-
-	/* we use an internal job to communicate with the native backend. plugin->job is used for client communication */
-	plugin->priv->internal_job = pk_backend_job_new ();
 
 	/* tell PK we might be able to handle these */
 	pk_backend_implement (plugin->backend, PK_ROLE_ENUM_GET_DETAILS);
@@ -939,10 +976,10 @@ void
 pk_plugin_destroy (PkPlugin *plugin)
 {
 	g_main_loop_unref (plugin->priv->loop);
-	pk_backend_stop_job (plugin->backend, plugin->priv->internal_job);
-	plugin->priv->internal_job = NULL;
-	if (plugin->priv->setup_settings != NULL)
-		g_object_unref (plugin->priv->setup_settings);
+	if (plugin->priv->internal_job != NULL) {
+		pk_backend_stop_job (plugin->backend, plugin->priv->internal_job);
+		plugin->priv->internal_job = NULL;
+	}
 	g_object_unref (plugin->priv->mgr);
 	g_object_unref (plugin->priv->backend_results);
 }
